/****************************************************************************
**
** This file is part of the LibreCAD project, a 2D CAD program
**
** Copyright (C) 2010 R. van Twisk (librecad@rvt.dds.nl)
** Copyright (C) 2001-2003 RibbonSoft. All rights reserved.
**
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software 
** Foundation and appearing in the file gpl-2.0.txt included in the
** packaging of this file.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
** This copyright notice MUST APPEAR in all copies of the script!  
**
**********************************************************************/

#ifndef RS_MAINWINDOWINTERFACE_H
#define RS_MAINWINDOWINTERFACE_H

#include "rs_document.h"

#include "rs_graphicview.h"

/**
 * Interface for main application windows.
 */
class RS_MainWindowInterface {

public:
	RS_MainWindowInterface() = default;
	virtual ~RS_MainWindowInterface() = default;

	virtual RS_GraphicView const* getGraphicView() const = 0;
	virtual RS_Document const* getDocument() const= 0;
	virtual RS_GraphicView* getGraphicView() = 0;
	virtual RS_Document* getDocument() = 0;

	virtual void createNewDocument(const QString& fileName = QString::null, RS_Document* doc=nullptr) = 0;

};

#endif

